@@ -22,6 +22,8 @@
 
 #define QT_FEATURE_no_direct_extern_access -1
 
+#define QT_FEATURE_x86intrin -1
+
 #define QT_FEATURE_sse2 -1
 
 #define QT_FEATURE_sse3 -1
@@ -112,6 +114,8 @@
 
 #define QT_FEATURE_libudev 1
 
+#define QT_FEATURE_openssl 1
+
 #define QT_FEATURE_dlopen 1
 
 #define QT_FEATURE_relocatable 1
