@@ -34,6 +34,8 @@
 
 #define QT_FEATURE_cxx2b -1
 
+#define QT_FEATURE_cxx2b -1
+
 #define QT_FEATURE_c99 1
 
 #define QT_FEATURE_c11 1
@@ -42,6 +44,10 @@
 
 #define QT_FEATURE_signaling_nan 1
 
+#define QT_FEATURE_wasm_simd128 -1
+
+#define QT_FEATURE_wasm_exceptions -1
+
 #define QT_FEATURE_zstd 1
 
 #define QT_FEATURE_thread 1
@@ -52,14 +58,21 @@
 
 #define QT_FEATURE_dbus 1
 
+#define QT_FEATURE_openssl_linked 1
 
-#define QT_VERSION_STR "6.4.3"
+#define QT_FEATURE_opensslv11 -1
+
+#define QT_FEATURE_opensslv30 1
+
+
+#define QT_VERSION_STR "6.5.0"
 
 #define QT_VERSION_MAJOR 6
 
-#define QT_VERSION_MINOR 4
+#define QT_VERSION_MINOR 5
 
-#define QT_VERSION_PATCH 3
+#define QT_VERSION_PATCH 0
 #define QT_LARGEFILE_SUPPORT 64
 #define QT_VISIBILITY_AVAILABLE
 #define QT_COMPILER_SUPPORTS_NEON 1
+#define QT_LINKED_OPENSSL
