@@ -18,7 +18,7 @@
 #include <algorithm>
 #include <initializer_list>
 #include <iterator>
-#include <memory>
+#include <QtCore/q20memory.h>
 #include <new>
 
 #include <string.h>
@@ -194,7 +194,7 @@ protected:
     {
         if (size() == capacity()) // ie. size() != 0
             growBy(prealloc, array, 1);
-        reference r = *new (end()) T(std::forward<Args>(args)...);
+        reference r = *q20::construct_at(end(), std::forward<Args>(args)...);
         ++s;
         return r;
     }
@@ -224,7 +224,7 @@ protected:
         }
         reallocate_impl(prealloc, array, sz, qMax(sz, capacity()));
         while (size() < sz) {
-            new (data() + size()) T(v);
+            q20::construct_at(data() + size(), v);
             ++s;
         }
     }
@@ -234,7 +234,7 @@ protected:
         if constexpr (QTypeInfo<T>::isComplex) {
             // call default constructor for new objects (which can throw)
             while (size() < sz) {
-                new (data() + size()) T;
+                q20::construct_at(data() + size());
                 ++s;
             }
         } else {
@@ -259,6 +259,7 @@ class QVarLengthArray
     friend class QVarLengthArray;
     using Base = QVLABase<T>;
     using Storage = QVLAStorage<sizeof(T), alignof(T), Prealloc>;
+    static_assert(Prealloc > 0, "QVarLengthArray Prealloc must be greater than 0.");
     static_assert(std::is_nothrow_destructible_v<T>, "Types with throwing destructors are not supported in Qt containers.");
     using Base::verify;
 
@@ -653,22 +654,22 @@ QVarLengthArray(InputIterator, InputIterator) -> QVarLengthArray<ValueType>;
 
 template <class T, qsizetype Prealloc>
 Q_INLINE_TEMPLATE QVarLengthArray<T, Prealloc>::QVarLengthArray(qsizetype asize)
-    : QVarLengthArray()
 {
-    Q_ASSERT_X(asize >= 0, "QVarLengthArray::QVarLengthArray(qsizetype)",
-               "Size must be greater than or equal to 0.");
-
-    // historically, this ctor worked for non-copyable/non-movable T, so keep it working, why not?
-    // resize(asize) // this requires a movable or copyable T, can't use, need to do it by hand
-
-    if (asize > Prealloc) {
-        this->ptr = malloc(asize * sizeof(T));
-        Q_CHECK_PTR(this->ptr);
-        this->a = asize;
-    }
-    if constexpr (QTypeInfo<T>::isComplex)
-        std::uninitialized_default_construct_n(data(), asize);
     this->s = asize;
+    Q_ASSERT_X(size() >= 0, "QVarLengthArray::QVarLengthArray()", "Size must be greater than or equal to 0.");
+    if (size() > Prealloc) {
+        this->ptr = malloc(size() * sizeof(T));
+        Q_CHECK_PTR(data());
+        this->a = size();
+    } else {
+        this->ptr = this->array;
+        this->a = Prealloc;
+    }
+    if constexpr (QTypeInfo<T>::isComplex) {
+        T *i = end();
+        while (i != begin())
+            q20::construct_at(--i);
+    }
 }
 
 template <class T>
@@ -852,12 +853,7 @@ Q_OUTOFLINE_TEMPLATE auto QVLABase<T>::emplace_impl(qsizetype prealloc, void *ar
     emplace_back_impl(prealloc, array, std::forward<Args>(args)...);
     const auto b = begin() + offset;
     const auto e = end();
-    if constexpr (QTypeInfo<T>::isRelocatable) {
-        auto cast = [](T *p) { return reinterpret_cast<uchar*>(p); };
-        std::rotate(cast(b), cast(e - 1), cast(e));
-    } else {
-        std::rotate(b, e - 1, e);
-    }
+    QtPrivate::q_rotate(b, e - 1, e);
     return b;
 }
 
@@ -866,28 +862,12 @@ Q_OUTOFLINE_TEMPLATE auto QVLABase<T>::insert_impl(qsizetype prealloc, void *arr
 {
     Q_ASSERT_X(isValidIterator(before), "QVarLengthArray::insert", "The specified const_iterator argument 'before' is invalid");
 
-    qsizetype offset = qsizetype(before - cbegin());
-    if (n != 0) {
-        const T copy(t); // `t` could alias an element in [begin(), end()[
-        resize_impl(prealloc, array, size() + n);
-        if constexpr (!QTypeInfo<T>::isRelocatable) {
-            T *b = begin() + offset;
-            T *j = end();
-            T *i = j - n;
-            while (i != b)
-                *--j = *--i;
-            i = b + n;
-            while (i != b)
-                *--i = copy;
-        } else {
-            T *b = begin() + offset;
-            T *i = b + n;
-            memmove(static_cast<void *>(i), static_cast<const void *>(b), (size() - offset - n) * sizeof(T));
-            while (i != b)
-                new (--i) T(copy);
-        }
-    }
-    return data() + offset;
+    const qsizetype offset = qsizetype(before - cbegin());
+    resize_impl(prealloc, array, size() + n, t);
+    const auto b = begin() + offset;
+    const auto e = end();
+    QtPrivate::q_rotate(b, e - n, e);
+    return b;
 }
 
 template <class T>
