@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtTest module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #ifndef QTESTCASE_H
 #define QTESTCASE_H
@@ -49,6 +13,7 @@
 #include <QtCore/qsharedpointer.h>
 #include <QtCore/qtemporarydir.h>
 #include <QtCore/qthread.h>
+#include <QtCore/qxpfunctional.h>
 
 #include <string.h>
 
@@ -90,6 +55,32 @@ do {\
         return;\
 } while (false)
 
+// A wrapper lambda is introduced to extend the lifetime of lhs and rhs in
+// case they are temporary objects.
+// We also use IILE to prevent potential name clashes and shadowing of variables
+// from user code. A drawback of the approach is that it looks ugly :(
+#define QCOMPARE_OP_IMPL(lhs, rhs, op, opId) \
+do { \
+    if (![](auto &&qt_lhs_arg, auto &&qt_rhs_arg) { \
+        /* assumes that op does not actually move from qt_{lhs, rhs}_arg */ \
+        return QTest::reportResult(std::forward<decltype(qt_lhs_arg)>(qt_lhs_arg) \
+                                   op \
+                                   std::forward<decltype(qt_rhs_arg)>(qt_rhs_arg), \
+                                   [&qt_lhs_arg] { return QTest::toString(qt_lhs_arg); }, \
+                                   [&qt_rhs_arg] { return QTest::toString(qt_rhs_arg); }, \
+                                   #lhs, #rhs, QTest::ComparisonOperation::opId, \
+                                   __FILE__, __LINE__); \
+    }(lhs, rhs)) { \
+        return; \
+    } \
+} while (false)
+
+#define QCOMPARE_EQ(lhs, rhs) QCOMPARE_OP_IMPL(lhs, rhs, ==, Equal)
+#define QCOMPARE_NE(lhs, rhs) QCOMPARE_OP_IMPL(lhs, rhs, !=, NotEqual)
+#define QCOMPARE_LT(lhs, rhs) QCOMPARE_OP_IMPL(lhs, rhs, <, LessThan)
+#define QCOMPARE_LE(lhs, rhs) QCOMPARE_OP_IMPL(lhs, rhs, <=, LessThanOrEqual)
+#define QCOMPARE_GT(lhs, rhs) QCOMPARE_OP_IMPL(lhs, rhs, >, GreaterThan)
+#define QCOMPARE_GE(lhs, rhs) QCOMPARE_OP_IMPL(lhs, rhs, >=, GreaterThanOrEqual)
 
 #ifndef QT_NO_EXCEPTIONS
 
@@ -157,9 +148,10 @@ inline void useVerifyThrowsException() {}
 /* Ideally we would adapt qWaitFor(), or a variant on it, to implement roughly
  * what the following provides as QTRY_LOOP_IMPL(); however, for now, the
  * reporting of how much to increase the timeout to (if within a factor of two)
- * on failure and the check for QTest::currentTestFailed() go beyond
- * qWaitFor(). (We no longer care about the bug in MSVC < 2017 that precluded
- * using qWaitFor() in the implementation here, see QTBUG-59096.)
+ * on failure and the check for (QTest::runningTest() &&
+ * QTest::currentTestResolved()) go beyond qWaitFor(). (We no longer care about
+ * the bug in MSVC < 2017 that precluded using qWaitFor() in the implementation
+ * here, see QTBUG-59096.)
  */
 
 // NB: not do {...} while (0) wrapped, as qt_test_i is accessed after it
@@ -168,50 +160,90 @@ inline void useVerifyThrowsException() {}
         QTest::qWait(0); \
     } \
     int qt_test_i = 0; \
-    for (; qt_test_i < timeoutValue && !(expr); qt_test_i += step) { \
+    for (; qt_test_i < timeoutValue && !(QTest::runningTest() && QTest::currentTestResolved()) \
+             && !(expr); qt_test_i += step) { \
         QTest::qWait(step); \
     }
+// Ends in a for-block, so doesn't want a following semicolon.
 
-#define QTRY_TIMEOUT_DEBUG_IMPL(expr, timeoutValue, step)\
-    if (!(expr)) { \
-        QTRY_LOOP_IMPL((expr), 2 * (timeoutValue), step);   \
-        if (expr) { \
-            QFAIL(qPrintable(QTest::Internal::formatTryTimeoutDebugMessage(u8"" #expr, timeoutValue, timeoutValue + qt_test_i))); \
+#define QTRY_TIMEOUT_DEBUG_IMPL(expr, timeoutValue, step) \
+    if (!(QTest::runningTest() && QTest::currentTestResolved()) && !(expr)) { \
+        QTRY_LOOP_IMPL(expr, 2 * (timeoutValue), step) \
+        if ((expr)) { \
+            QFAIL(qPrintable(QTest::Internal::formatTryTimeoutDebugMessage(\
+                                 u8"" #expr, timeoutValue, timeoutValue + qt_test_i))); \
         } \
     }
 
 #define QTRY_IMPL(expr, timeout)\
     const int qt_test_step = timeout < 350 ? timeout / 7 + 1 : 50; \
     const int qt_test_timeoutValue = timeout; \
-    { QTRY_LOOP_IMPL(QTest::currentTestFailed() || (expr), qt_test_timeoutValue, qt_test_step); } \
-    QTRY_TIMEOUT_DEBUG_IMPL(QTest::currentTestFailed() || (expr), qt_test_timeoutValue, qt_test_step)
+    { QTRY_LOOP_IMPL(expr, qt_test_timeoutValue, qt_test_step) } \
+    QTRY_TIMEOUT_DEBUG_IMPL(expr, qt_test_timeoutValue, qt_test_step)
+// Ends with an if-block, so doesn't want a following semicolon.
 
 // Will try to wait for the expression to become true while allowing event processing
 #define QTRY_VERIFY_WITH_TIMEOUT(expr, timeout) \
 do { \
-    QTRY_IMPL((expr), timeout);\
+    QTRY_IMPL(expr, timeout) \
     QVERIFY(expr); \
 } while (false)
 
-#define QTRY_VERIFY(expr) QTRY_VERIFY_WITH_TIMEOUT((expr), 5000)
+#define QTRY_VERIFY(expr) QTRY_VERIFY_WITH_TIMEOUT(expr, 5000)
 
 // Will try to wait for the expression to become true while allowing event processing
 #define QTRY_VERIFY2_WITH_TIMEOUT(expr, messageExpression, timeout) \
 do { \
-    QTRY_IMPL((expr), timeout);\
+    QTRY_IMPL(expr, timeout) \
     QVERIFY2(expr, messageExpression); \
 } while (false)
 
-#define QTRY_VERIFY2(expr, messageExpression) QTRY_VERIFY2_WITH_TIMEOUT((expr), (messageExpression), 5000)
+#define QTRY_VERIFY2(expr, messageExpression) QTRY_VERIFY2_WITH_TIMEOUT(expr, messageExpression, 5000)
 
 // Will try to wait for the comparison to become successful while allowing event processing
 #define QTRY_COMPARE_WITH_TIMEOUT(expr, expected, timeout) \
 do { \
-    QTRY_IMPL(((expr) == (expected)), timeout);\
-    QCOMPARE((expr), expected); \
+    QTRY_IMPL((expr) == (expected), timeout) \
+    QCOMPARE(expr, expected); \
 } while (false)
 
-#define QTRY_COMPARE(expr, expected) QTRY_COMPARE_WITH_TIMEOUT((expr), expected, 5000)
+#define QTRY_COMPARE(expr, expected) QTRY_COMPARE_WITH_TIMEOUT(expr, expected, 5000)
+
+#define QTRY_COMPARE_OP_WITH_TIMEOUT_IMPL(left, right, op, opId, timeout) \
+do { \
+    QTRY_IMPL(((left) op (right)), timeout) \
+    QCOMPARE_OP_IMPL(left, right, op, opId); \
+} while (false)
+
+#define QTRY_COMPARE_EQ_WITH_TIMEOUT(left, right, timeout) \
+    QTRY_COMPARE_OP_WITH_TIMEOUT_IMPL(left, right, ==, Equal, timeout)
+
+#define QTRY_COMPARE_EQ(left, right) QTRY_COMPARE_EQ_WITH_TIMEOUT(left, right, 5000)
+
+#define QTRY_COMPARE_NE_WITH_TIMEOUT(left, right, timeout) \
+    QTRY_COMPARE_OP_WITH_TIMEOUT_IMPL(left, right, !=, NotEqual, timeout)
+
+#define QTRY_COMPARE_NE(left, right) QTRY_COMPARE_NE_WITH_TIMEOUT(left, right, 5000)
+
+#define QTRY_COMPARE_LT_WITH_TIMEOUT(left, right, timeout) \
+    QTRY_COMPARE_OP_WITH_TIMEOUT_IMPL(left, right, <, LessThan, timeout)
+
+#define QTRY_COMPARE_LT(left, right) QTRY_COMPARE_LT_WITH_TIMEOUT(left, right, 5000)
+
+#define QTRY_COMPARE_LE_WITH_TIMEOUT(left, right, timeout) \
+    QTRY_COMPARE_OP_WITH_TIMEOUT_IMPL(left, right, <=, LessThanOrEqual, timeout)
+
+#define QTRY_COMPARE_LE(left, right) QTRY_COMPARE_LE_WITH_TIMEOUT(left, right, 5000)
+
+#define QTRY_COMPARE_GT_WITH_TIMEOUT(left, right, timeout) \
+    QTRY_COMPARE_OP_WITH_TIMEOUT_IMPL(left, right, >, GreaterThan, timeout)
+
+#define QTRY_COMPARE_GT(left, right) QTRY_COMPARE_GT_WITH_TIMEOUT(left, right, 5000)
+
+#define QTRY_COMPARE_GE_WITH_TIMEOUT(left, right, timeout) \
+    QTRY_COMPARE_OP_WITH_TIMEOUT_IMPL(left, right, >=, GreaterThanOrEqual, timeout)
+
+#define QTRY_COMPARE_GE(left, right) QTRY_COMPARE_GE_WITH_TIMEOUT(left, right, 5000)
 
 #define QSKIP_INTERNAL(statement) \
 do {\
@@ -280,9 +312,18 @@ namespace QTest
         return qstrdup(QByteArray::number(static_cast<std::underlying_type_t<T>>(e)).constData());
     }
 
-    template <typename T> // Fallback
-    inline typename std::enable_if<!QtPrivate::IsQEnumHelper<T>::Value && !std::is_enum_v<T>, char*>::type toString(const T &)
+    template <typename T> // Fallback; for built-in types debug streaming must be possible
+    inline typename std::enable_if<!QtPrivate::IsQEnumHelper<T>::Value && !std::is_enum_v<T>, char *>::type toString(const T &t)
     {
+#ifndef QT_NO_DEBUG_STREAM
+        if constexpr (QTypeTraits::has_ostream_operator_v<QDebug, T>) {
+            return qstrdup(QDebug::toString(t).toUtf8().constData());
+        } else {
+            static_assert(!QMetaTypeId2<T>::IsBuiltIn,
+                        "Built-in type must implement debug streaming operator "
+                        "or provide QTest::toString specialization");
+        }
+#endif
         return nullptr;
     }
 
@@ -319,8 +360,8 @@ namespace QTest
     template <class... Types>
     inline char *toString(const std::tuple<Types...> &tuple);
 
-    Q_TESTLIB_EXPORT char *toHexRepresentation(const char *ba, int length);
-    Q_TESTLIB_EXPORT char *toPrettyCString(const char *unicode, int length);
+    Q_TESTLIB_EXPORT char *toHexRepresentation(const char *ba, qsizetype length);
+    Q_TESTLIB_EXPORT char *toPrettyCString(const char *unicode, qsizetype length);
     Q_TESTLIB_EXPORT char *toPrettyUnicode(QStringView string);
     Q_TESTLIB_EXPORT char *toString(const char *);
     Q_TESTLIB_EXPORT char *toString(const volatile void *);
@@ -334,6 +375,11 @@ namespace QTest
     Q_TESTLIB_EXPORT int qExec(QObject *testObject, int argc = 0, char **argv = nullptr);
     Q_TESTLIB_EXPORT int qExec(QObject *testObject, const QStringList &arguments);
 
+#if QT_CONFIG(batch_test_support) || defined(Q_QDOC)
+    using TestEntryFunction = int (*)(int, char **);
+    Q_TESTLIB_EXPORT void qRegisterTestCase(const QString &name, TestEntryFunction entryFunction);
+#endif  // QT_CONFIG(batch_test_support)
+
     Q_TESTLIB_EXPORT void setMainSourcePath(const char *file, const char *builddir = nullptr);
 
     Q_TESTLIB_EXPORT bool qVerify(bool statement, const char *statementStr, const char *description,
@@ -374,14 +420,32 @@ namespace QTest
     Q_TESTLIB_EXPORT const char *currentTestFunction();
     Q_TESTLIB_EXPORT const char *currentDataTag();
     Q_TESTLIB_EXPORT bool currentTestFailed();
+    Q_TESTLIB_EXPORT bool currentTestResolved();
+    Q_TESTLIB_EXPORT bool runningTest(); // Internal, for use by macros and QTestEventLoop.
 
     Q_TESTLIB_EXPORT Qt::Key asciiToKey(char ascii);
     Q_TESTLIB_EXPORT char keyToAscii(Qt::Key key);
 
+    // ### TODO: remove QTestResult::compare() overload that takes char * values
+    // when this overload is removed.
+#if QT_DEPRECATED_SINCE(6, 4)
+    QT_DEPRECATED_VERSION_X_6_4("use an overload that takes function_ref as parameters, "
+                                "or an overload that takes only failure message, if you "
+                                "do not need to stringify the values")
     Q_TESTLIB_EXPORT bool compare_helper(bool success, const char *failureMsg,
-                                         char *val1, char *val2,
+                                         char *actualVal, char *expectedVal,
                                          const char *actual, const char *expected,
                                          const char *file, int line);
+#endif // QT_DEPRECATED_SINCE(6, 4)
+    Q_TESTLIB_EXPORT bool compare_helper(bool success, const char *failureMsg,
+                                         qxp::function_ref<const char*()> actualVal,
+                                         qxp::function_ref<const char*()> expectedVal,
+                                         const char *actual, const char *expected,
+                                         const char *file, int line);
+    Q_TESTLIB_EXPORT bool compare_helper(bool success, const char *failureMsg,
+                                         const char *actual, const char *expected,
+                                         const char *file, int line);
+
     Q_TESTLIB_EXPORT void addColumnInternal(int id, const char *name);
 
     template <typename T>
@@ -394,17 +458,6 @@ namespace QTest
     Q_TESTLIB_EXPORT QTestData &newRow(const char *dataTag);
     Q_TESTLIB_EXPORT QTestData &addRow(const char *format, ...) Q_ATTRIBUTE_FORMAT_PRINTF(1, 2);
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    // kept after adding implementation of <T1, T2> out of paranoia:
-    template <typename T>
-    inline bool qCompare(T const &t1, T const &t2, const char *actual, const char *expected,
-                        const char *file, int line)
-    {
-        return compare_helper(t1 == t2, "Compared values are not the same",
-                              toString(t1), toString(t2), actual, expected, file, line);
-    }
-#endif
-
     Q_TESTLIB_EXPORT bool qCompare(qfloat16 const &t1, qfloat16 const &t2,
                     const char *actual, const char *expected, const char *file, int line);
 
@@ -428,10 +481,10 @@ namespace QTest
     Q_TESTLIB_EXPORT bool qCompare(QStringView t1, QStringView t2,
                                    const char *actual, const char *expected,
                                    const char *file, int line);
-    Q_TESTLIB_EXPORT bool qCompare(QStringView t1, const QLatin1String &t2,
+    Q_TESTLIB_EXPORT bool qCompare(QStringView t1, const QLatin1StringView &t2,
                                    const char *actual, const char *expected,
                                    const char *file, int line);
-    Q_TESTLIB_EXPORT bool qCompare(const QLatin1String &t1, QStringView t2,
+    Q_TESTLIB_EXPORT bool qCompare(const QLatin1StringView &t1, QStringView t2,
                                    const char *actual, const char *expected,
                                    const char *file, int line);
     inline bool qCompare(const QString &t1, const QString &t2,
@@ -440,13 +493,13 @@ namespace QTest
     {
         return qCompare(QStringView(t1), QStringView(t2), actual, expected, file, line);
     }
-    inline bool qCompare(const QString &t1, const QLatin1String &t2,
+    inline bool qCompare(const QString &t1, const QLatin1StringView &t2,
                          const char *actual, const char *expected,
                          const char *file, int line)
     {
         return qCompare(QStringView(t1), t2, actual, expected, file, line);
     }
-    inline bool qCompare(const QLatin1String &t1, const QString &t2,
+    inline bool qCompare(const QLatin1StringView &t1, const QString &t2,
                          const char *actual, const char *expected,
                          const char *file, int line)
     {
@@ -457,42 +510,48 @@ namespace QTest
                                    const char *expected, const char *file, int line)
     {
         return compare_helper(t1 == t2, "Compared pointers are not the same",
-                              toString(t1), toString(t2), actual, expected, file, line);
+                              [t1] { return toString(t1); }, [t2] { return toString(t2); },
+                              actual, expected, file, line);
     }
 
     inline bool compare_ptr_helper(const volatile QObject *t1, const volatile QObject *t2, const char *actual,
                                    const char *expected, const char *file, int line)
     {
         return compare_helper(t1 == t2, "Compared QObject pointers are not the same",
-                              toString(t1), toString(t2), actual, expected, file, line);
+                              [t1] { return toString(t1); }, [t2] { return toString(t2); },
+                              actual, expected, file, line);
     }
 
     inline bool compare_ptr_helper(const volatile QObject *t1, std::nullptr_t, const char *actual,
                                    const char *expected, const char *file, int line)
     {
         return compare_helper(t1 == nullptr, "Compared QObject pointers are not the same",
-                              toString(t1), toString(nullptr), actual, expected, file, line);
+                              [t1] { return toString(t1); }, [] { return toString(nullptr); },
+                              actual, expected, file, line);
     }
 
     inline bool compare_ptr_helper(std::nullptr_t, const volatile QObject *t2, const char *actual,
                                    const char *expected, const char *file, int line)
     {
         return compare_helper(nullptr == t2, "Compared QObject pointers are not the same",
-                              toString(nullptr), toString(t2), actual, expected, file, line);
+                              [] { return toString(nullptr); }, [t2] { return toString(t2); },
+                              actual, expected, file, line);
     }
 
     inline bool compare_ptr_helper(const volatile void *t1, std::nullptr_t, const char *actual,
                                    const char *expected, const char *file, int line)
     {
         return compare_helper(t1 == nullptr, "Compared pointers are not the same",
-                              toString(t1), toString(nullptr), actual, expected, file, line);
+                              [t1] { return toString(t1); }, [] { return toString(nullptr); },
+                              actual, expected, file, line);
     }
 
     inline bool compare_ptr_helper(std::nullptr_t, const volatile void *t2, const char *actual,
                                    const char *expected, const char *file, int line)
     {
         return compare_helper(nullptr == t2, "Compared pointers are not the same",
-                              toString(nullptr), toString(t2), actual, expected, file, line);
+                              [] { return toString(nullptr); }, [t2] { return toString(t2); },
+                              actual, expected, file, line);
     }
 
     Q_TESTLIB_EXPORT bool compare_string_helper(const char *t1, const char *t2, const char *actual,
@@ -524,7 +583,8 @@ namespace QTest
                          const char *file, int line)
     {
         return compare_helper(t1 == t2, "Compared values are not the same",
-                              toString(t1), toString(t2), actual, expected, file, line);
+                              [&t1] { return toString(t1); }, [&t2] { return toString(t2); },
+                              actual, expected, file, line);
     }
 
     inline bool qCompare(double const &t1, float const &t2, const char *actual,
@@ -610,6 +670,11 @@ namespace QTest
         return qCompare(actual, *static_cast<const T *>(QTest::qElementData(elementName,
                        qMetaTypeId<T>())), actualStr, expected, file, line);
     }
+
+    Q_TESTLIB_EXPORT bool reportResult(bool success, qxp::function_ref<const char*()> lhs,
+                                       qxp::function_ref<const char*()> rhs,
+                                       const char *lhsExpr, const char *rhsExpr,
+                                       ComparisonOperation op, const char *file, int line);
 }
 
 #undef QTEST_COMPARE_DECL
