@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtWidgets module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #ifndef QWIDGET_P_H
 #define QWIDGET_P_H
@@ -61,7 +25,6 @@
 #include "QtCore/qset.h"
 #include "QtGui/qregion.h"
 #include "QtGui/qinputmethod.h"
-#include "QtGui/qopengl.h"
 #include "QtGui/qsurfaceformat.h"
 #include "QtGui/qscreen.h"
 #include "QtWidgets/qsizepolicy.h"
@@ -77,6 +40,7 @@
 #endif
 #include <private/qgesture_p.h>
 #include <qpa/qplatformbackingstore.h>
+#include <QtGui/private/qbackingstorerhisupport_p.h>
 
 #include <vector>
 #include <memory>
@@ -94,7 +58,6 @@ class QPixmap;
 class QWidgetRepaintManager;
 class QGraphicsProxyWidget;
 class QWidgetItemV2;
-class QOpenGLContext;
 
 class QStyle;
 
@@ -130,9 +93,6 @@ struct QTLWExtra {
     QBackingStore *backingStore;
     QPainter *sharedPainter;
     QWidgetWindow *window;
-#ifndef QT_NO_OPENGL
-    mutable std::unique_ptr<QOpenGLContext> shareContext;
-#endif
 
     // Implicit pointers (shared_null).
     QString caption; // widget caption
@@ -149,9 +109,7 @@ struct QTLWExtra {
     Qt::WindowFlags savedFlags; // Save widget flags while showing fullscreen
     QScreen *initialScreen; // Screen when passing a QDesktop[Screen]Widget as parent.
 
-#ifndef QT_NO_OPENGL
     std::vector<std::unique_ptr<QPlatformTextureList>> widgetTextures;
-#endif
 
     // *************************** Cross-platform bit fields ****************************
     uint opacity : 8;
@@ -252,6 +210,9 @@ public:
     static QWidgetPrivate *get(QWidget *w) { return w->d_func(); }
     static const QWidgetPrivate *get(const QWidget *w) { return w->d_func(); }
 
+    static void checkRestoredGeometry(const QRect &availableGeometry, QRect *restoredGeometry,
+                                      int frameHeight);
+
     QWExtra *extraData() const;
     QTLWExtra *topData() const;
     QTLWExtra *maybeTopData() const;
@@ -617,12 +578,17 @@ public:
     inline QRect mapFromWS(const QRect &r) const
     { return r.translated(data.wrect.topLeft()); }
 
-    QOpenGLContext *shareContext() const;
-
     virtual QObject *focusObject() { return nullptr; }
 
-#ifndef QT_NO_OPENGL
-    virtual GLuint textureId() const { return 0; }
+    virtual QPlatformBackingStoreRhiConfig rhiConfig() const { return {}; }
+
+    // Note that textureRight may be null, as it's only used in stereoscopic rendering
+    struct TextureData {
+        QRhiTexture *textureLeft = nullptr;
+        QRhiTexture *textureRight = nullptr;
+    };
+
+    virtual TextureData texture() const { return {}; }
     virtual QPlatformTextureList::Flags textureListFlags() {
         Q_Q(QWidget);
         return q->testAttribute(Qt::WA_AlwaysStackOnTop)
@@ -657,10 +623,16 @@ public:
     virtual void resizeViewportFramebuffer() { }
     // Called after each paint event.
     virtual void resolveSamples() { }
-#endif
+
+    // These two are used in QGraphicsView for supporting stereoscopic rendering with a
+    // QOpenGLWidget viewport.
+    virtual bool isStereoEnabled() { return false; } // Called in QGraphicsView::setupViewport
+    virtual bool toggleStereoTargetBuffer() { return false; } // Called in QGraphicsView::paintEvent
 
     static void setWidgetParentHelper(QObject *widgetAsObject, QObject *newParent);
 
+    std::string flagsForDumping() const override;
+
     // Variables.
     // Regular pointers (keep them together to avoid gaps on 64 bit architectures).
     std::unique_ptr<QWExtra> extra;
@@ -699,7 +671,7 @@ public:
 #if QT_CONFIG(whatsthis)
     QString whatsThis;
 #endif
-#ifndef QT_NO_ACCESSIBILITY
+#if QT_CONFIG(accessibility)
     QString accessibleName;
     QString accessibleDescription;
 #endif
@@ -748,10 +720,8 @@ public:
 #ifndef QT_NO_IM
     uint inheritsInputMethodHints : 1;
 #endif
-#ifndef QT_NO_OPENGL
     uint renderToTextureReallyDirty : 1;
-    uint renderToTextureComposeActive : 1;
-#endif
+    uint usesRhiFlush : 1;
     uint childrenHiddenByWState : 1;
     uint childrenShownByExpose : 1;
 
